// BinaryTree class; stores a binary tree.
//
// CONSTRUCTION: with (a) no parameters or (b) an object to
//    be placed in the root of a one-element tree.
//
// *******************PUBLIC OPERATIONS**********************
// Various tree traversals, size, height, isEmpty, makeEmpty.
// Also, the following tricky method:
// void merge( Object root, BinaryTree t1, BinaryTree t2 )
//                        --> Construct a new tree
// *******************ERRORS*********************************
// Error message printed for illegal merges.

/**
 * BinaryTree class that illustrates the calling of
 * BinaryNode recursive routines and merge.
 */
public class BinaryTree
{
    public BinaryTree( )
    {
        root = null;
    }

    public BinaryTree( Object rootItem )
    {
        root = new BinaryNode( rootItem, null, null );
    }

    public void printPreOrder( )
    {
        if( root != null )
            root.printPreOrder( );
    }

    public void printInOrder( )
    {
        if( root != null )
           root.printInOrder( );
    }

    public void printPostOrder( )
    {
        if( root != null )
           root.printPostOrder( );
    }

	public String inOrder()
	{
		return inOrderNode(root);
	}
	
	public String inOrderNode(BinaryNode node)
	{
		String InOrderString = new String();
		InOrderString=(String)node.getElement();
		if(node.getLeft()!=null)
			InOrderString = InOrderString.concat(inOrderNode(node.getLeft()));
		if(node.getRight()!=null)
			InOrderString = InOrderString.concat(inOrderNode(node.getRight()));;
		return InOrderString;
	}	

	public String CustomOrder()
	{
		return inCustomOrder(root, 0);
	}
	
	public String inCustomOrder(BinaryNode node, int level)
	{
		String InOrderString = new String();
		if(node.getLeft()!=null)
			InOrderString = InOrderString.concat(inCustomOrder(node.getLeft(), level+1));
		InOrderString = InOrderString.concat(InOrderString.valueOf(level));
		InOrderString = InOrderString.concat((String)node.getElement());
		if(node.getRight()!=null)
			InOrderString = InOrderString.concat(inCustomOrder(node.getRight(), level+1));;
		return InOrderString;
	}

    public void makeEmpty( )
    {
        root = null;
    }

    public boolean isEmpty( )
    {
        return root == null;
    }
    
    public BinaryNode ReturnRoot()
    {
    	return root;	
    }
    /**
     * Merge routine for BinaryTree class.
     * Forms a new tree from rootItem, t1 and t2.
     * Does not allow t1 and t2 to be the same.
     * Correctly handles other aliasing conditions.
     */
    public void merge( Object rootItem, BinaryTree t1, BinaryTree t2 )
    {
        if( t1.root == t2.root && t1.root != null )
        {
            System.err.println( "leftTree==rightTree; merge aborted" );
            return;
        }

            // Allocate new node
        root = new BinaryNode( rootItem, t1.root, t2.root );

            // Ensure that every node is in one tree
        if( this != t1 )
            t1.root = null;
        if( this != t2 )
            t2.root = null;
    }

    public int size( )
    {
        return BinaryNode.size( root );
    }

    public int height( )
    {
        return BinaryNode.height( root );
    }

    public BinaryNode getRoot( )
    {
        return root;
    }
    
    private BinaryNode root;

    static public void main( String [ ] args )
    {
        BinaryTree NewTree = new BinaryTree();
        BuildTree BuildTreeVar = new BuildTree();
        NewTree = BuildTreeVar.preOrderBuild("ETORHIDCANSLFMGYU", "2R2R0LR0RL202LR00");
        DisplayableBinaryTree DispTree = new DisplayableBinaryTree(NewTree, 1000,1000);
        DispTree.display();
    }
}