import java.io.*;
import java.util.*;

/**
 * A WordListPrinter produces a sorted list of all words from an input file.
 *
 * @author Claude Anderson, CS 230.
 */

public abstract class WordListPrinter {

    protected Collection theList;
    protected BufferedReader in;
    protected PrintStream out;

    /**
     * Constructs a new WordListPrinter with the specified input and
     * output streams.
     *
     * @param _in Input stream
     * @param _out Output stream
     */

    public WordListPrinter(BufferedReader _in, PrintStream _out) {
        in  = _in;
        out = _out;
    }

    /**
     * Reads all of the words from the input stream provided to the
     * constructor and prints a sorted list of unique words
     * to the output stream also provided to the constructor 
     */ 

    public abstract void outputWordList();

    /**
     * Outputs the sorted collection to the output stream provided to
     * the constructor.
     */

    protected void iterateAndOutput() {
        int i = 0;
        Iterator itr = theList.iterator();
        while (itr.hasNext())
            out.print(itr.next() + " ");
        out.print("\n");
    }
    
    // Feel free to add other abstract or concrete methods. 
    protected final String readOneLine()
    {
    	String oneLine = null;
    	
    	try
    	{
    		oneLine = in.readLine();
    	}
    	catch(IOException e)
    	{
    		// ugggh, I hope this doesn't happen.  We're not supposed to do output
    		// though.
    	}
    	
    	return oneLine;
	}
}